@@ -7,11 +7,12 @@ import re
 import shutil
 from pynetdicom import AE, evt, AllStoragePresentationContexts, StoragePresentationContexts
 import time
+import pkg_resources
 
 log = logging.getLogger(__name__)
 LOGFORMAT = "%(levelname)s %(asctime)s [%(filename)-15s:%(lineno)-4s][%(funcName)-30s]\t%(message)s"
 LOGFORMAT_console = "[%(funcName)-25s]\t%(message).40s ..."
-logging.basicConfig(level=logging.INFO, format=LOGFORMAT, datefmt='%H:%M:%S')
+logging.basicConfig(level=logging.ERROR, format=LOGFORMAT, datefmt='%H:%M:%S')
 
 
 class pyconquest:
@@ -28,26 +29,30 @@ class pyconquest:
     __prev_patientid = ''
     __db_design = {}
     __truncate_colnames = True
+    __version__ = pkg_resources.get_distribution("pyconquest").version
     data_directory = ''
     sql_inifile_name = ''
     database_filename = ''
     conn_pacs = ''
 
     def __init__(self, data_directory='data', sql_inifile_name='dicom.sql', database_filename='conquest.db',
-                 connect_and_read_sql=True, loglevel='INFO'):
+                 connect_and_read_sql=True, loglevel='ERROR'):
         self.data_directory = data_directory
         self.sql_inifile_name = sql_inifile_name
         self.database_filename = database_filename
 
         if loglevel == 'ERROR':
             log.level = logging.ERROR
+        elif loglevel == 'DEBUG':
+            log.level = logging.DEBUG
+        elif loglevel == 'INFO':
+            log.level = logging.INFO
 
         if connect_and_read_sql:
             self.connect_db()
             self.read_conquest_sql_inifile(self.sql_inifile_name)
-
-        log.info("Created pyconquest with dir:({}); db:({});ini:({}) "
-                 .format(data_directory, database_filename, sql_inifile_name))
+        log.info("Created pyconquest({}) with dir:({}); db:({});ini:({}) "
+                 .format(self.__version__, data_directory, database_filename, sql_inifile_name))
 
     #
     #   some general multi purpose database interaction routines
@@ -70,6 +75,8 @@ class pyconquest:
             data = cursor.fetchall()
             cols = cursor.description
             query_result = [dict(line) for line in [zip([column[0] for column in cols], row) for row in data]]
+            log.debug('Query : '+query)
+            log.debug('Result: ' + str(query_result))
         except Exception as e:
             log.error('exception ' + str(e) + '\nencountered in execution of db query: ' + query)
         return query_result
@@ -87,7 +94,7 @@ class pyconquest:
         query = 'SELECT * FROM ' + tablename + ' WHERE ' + colname + ' = \"' + value.strip() + '\"'
         cur.execute(query)
         row = cur.fetchone()
-        if row == None:
+        if row is None:
             return False
         else:
             return True
@@ -99,8 +106,8 @@ class pyconquest:
         sql = """INSERT INTO %s %s VALUES %s""" % (table, columns_string, values_string)
         return sql
 
-    def create_buildquery(self, table, myDict):
-        columns_string = (' character varying(128),'.join(myDict.keys()) + ' character varying(128)').replace("-", "_")
+    def create_buildquery(self, table, mydict):
+        columns_string = (' character varying(128),'.join(mydict.keys()) + ' character varying(128)').replace("-", "_")
         sql = """CREATE TABLE %s (%s)
                  """ % (table, columns_string)
         return sql
@@ -111,17 +118,34 @@ class pyconquest:
 
     def __convert_listvalues_to_conquest_style(self, Dict):
         for key, val in Dict.items():
-            if str(val).startswith("[") == True and key not in self.__extra_imagetable_columns:
+            if str(val).startswith("[") is True and key not in self.__extra_imagetable_columns:
                 val = str(val).replace('[', '').replace(']', '').replace(',', "\\")
                 Dict[key] = val
         return Dict
 
+    def add_column_to_database(self, tablename, column_definition):
+        # add an individual column to the database
+        # example of column definition : ['0x0020', '0x000e', 'SeriesInst']
+        if not isinstance(column_definition[0], list):
+            column_definition = [column_definition]
+
+        for col in column_definition:
+            try:
+                self.__db_design[tablename].append(col)
+                log.info('Added column {} to table {}'.format(col, tablename))
+            except Exception:
+                log.error('failed to add column {} to table {}'.format(col, tablename))
+
     def read_conquest_sql_inifile(self, filename):
         # reads in the conquest style .sql file where the db is defined
-        with open(filename) as file:
-            lines = file.readlines()
-            lines = [line.rstrip() for line in lines]
-        file.close()
+        if os.path.exists(filename):
+            with open(filename) as file:
+                lines = file.readlines()
+                lines = [line.rstrip() for line in lines]
+            file.close()
+        else:
+            self.__set_default_database()
+            return
 
         for line in lines:
             # comment lines
@@ -142,10 +166,10 @@ class pyconquest:
 
             stripped_line = re.sub("\s+", ' ', line)
             if stripped_line.startswith(" {"):
-                col = (line.replace("\t{", "").split(","))
+                col = (line.replace("\t{", "").replace("}", "").replace("{", '').replace(" ", "").split(","))
                 if self.__truncate_colnames:
                     col[2] = col[2].replace('"', '').replace(' ', '')[0:10]
-                list_of_rows.append(col)
+                list_of_rows.append(col[0:3])
 
     def __create_tabledict(self, tablename, ds):
         tabledict = {}
@@ -200,6 +224,7 @@ class pyconquest:
                         self.execute_db_query(query)
 
     def create_standard_dicom_tables(self):
+
         for tablename in self.__db_design:
             tablelist = self.__db_design[tablename]
             colnames = {}
@@ -242,11 +267,12 @@ class pyconquest:
             path = "{}/{}".format(self.data_directory, patientid)
             if not os.path.exists(path):
                 os.makedirs(path)
-                log.info("Directory ", path, " Created ")
+                log.info("Directory " + path + " Created ")
 
             shutil.copy(filename, target_filename)
             os.remove(filename)
             self.write_tags(ds, target_filename)
+            log.info('stored file : '+filename+' in database at location: '+target_filename)
         except Exception as e:
             log.error('Exception encountered in store_dicom_file: ' + str(e))
 
@@ -282,7 +308,7 @@ class pyconquest:
     def copy_dicom_files_to_dest(self, seriesuid=None, studyuid=None, patientid=None, destination='', CreateDir=True):
         # copies all dicom files belonging to a series/study/patient to
 
-        if not seriesuid == None:
+        if not seriesuid is None:
             query = "select Objectfile from dicomimages where seriesinst=\"{}\"".format(seriesuid)
             return_list = self.execute_db_query(query)
         else:
@@ -292,7 +318,7 @@ class pyconquest:
         if return_list and CreateDir:
             if not os.path.exists(destination):
                 os.makedirs(destination)
-                log.info("Directory ", destination, " Created ")
+                log.info("Directory "+destination+ " Created ")
 
         for row in return_list:
             filename = "{}/{}".format(self.data_directory, row['Objectfile'])
@@ -381,17 +407,17 @@ class pyconquest:
         path = "{}/{}".format(self.data_directory, patientid)
         if not os.path.exists(path):
             os.makedirs(path)
-            log.info("Directory ", path, " Created ")
+            log.info("Directory " + path + " Created ")
 
         # Save the dataset using the SOP Instance UID as the filename
         ds.save_as(filename, write_like_original=False)
-        log.info('dicom saved to file : ', filename)
+        log.info('dicom saved to file : ' + filename)
 
-        if not args == None:
+        if not args is None:
             xtra_filename = "{}/{}.dcm".format(args, ds.SOPInstanceUID)
             try:
                 ds.save_as(xtra_filename, write_like_original=False)
-                log.info('dicom saved to file : ', xtra_filename)
+                log.info('dicom saved to file : ' + xtra_filename)
             except Exception as e:
                 log.error(e)
 
@@ -415,9 +441,124 @@ class pyconquest:
                 ",(select count(*) from dicomseries where seriespat=patientid and modality=\'CT\') as nrCT" \
                 ",(select count(*) from dicomseries where seriespat=patientid and modality=\'MR\') as nrMR" \
                 ",(select count(*) from dicomseries where seriespat=patientid and modality=\'PT\') as nrPT" \
-                ",(select count(*) from dicomseries where seriespat=patientid and modality=\'RTSTRUCT\') as nrRTSTRUCT" \
+                ",(select count(*) from dicomseries where seriespat=patientid and modality=\'RTSTRUCT\') as nrRTSTRUCT"\
                 ",(select count(*) from dicomseries where seriespat=patientid and modality=\'RTDOSE\') as nrRTDOSE" \
                 ",(select count(*) from dicomseries where seriespat=patientid and modality=\'RTPLAN\') as nrRTPLAN" \
                 " from dicompatients order by {}".format(orderby)
         result = self.execute_db_query(query)
         return result
+
+    def __set_default_database(self):
+        log.info('Using default database layout,specify .sql file during instance creation to change this')
+        self.__db_design = \
+            {'DICOMpatients':
+                 [['0x0010', '0x0020', 'PatientID'],
+                  ['0x0010', '0x0010', 'PatientNam'],
+                  ['0x0010', '0x0030', 'PatientBir'],
+                  ['0x0010', '0x0040', 'PatientSex']],
+             'DICOMstudies':
+                 [['0x0020', '0x000d', 'StudyInsta'],
+                  ['0x0008', '0x0020', 'StudyDate'],
+                  ['0x0008', '0x0030', 'StudyTime'],
+                  ['0x0020', '0x0010', 'StudyID'],
+                  ['0x0008', '0x1030', 'StudyDescr'],
+                  ['0x0008', '0x0050', 'AccessionN'],
+                  ['0x0008', '0x0090', 'ReferPhysi'],
+                  ['0x0010', '0x1010', 'PatientsAg'],
+                  ['0x0010', '0x1030', 'PatientsWe'],
+                  ['0x0008', '0x0061', 'StudyModal'],
+                  ['0x0010', '0x0010', 'PatientNam'],
+                  ['0x0010', '0x0030', 'PatientBir'],
+                  ['0x0010', '0x0040', 'PatientSex'],
+                  ['0x0008', '0x1070', 'OperatorsN'],
+                  ['0x0010', '0x0020', 'PatientID']],
+             'DICOMseries':
+                 [['0x0020', '0x000e', 'SeriesInst'],
+                  ['0x0020', '0x0011', 'SeriesNumb'],
+                  ['0x0008', '0x0021', 'SeriesDate'],
+                  ['0x0008', '0x0031', 'SeriesTime'],
+                  ['0x0008', '0x103e', 'SeriesDesc'],
+                  ['0x0008', '0x0060', 'Modality'],
+                  ['0x0008', '0x1090', 'ManModelNa'],
+                  ['0x0008', '0x1155', 'Referenced'],
+                  ['0x0018', '0x5100', 'PatientPos'],
+                  ['0x0018', '0x0010', 'ContrastBo'],
+                  ['0x0008', '0x0070', 'Manufactur'],
+                  ['0x0018', '0x0015', 'BodyPartEx'],
+                  ['0x0018', '0x1030', 'ProtocolNa'],
+                  ['0x0008', '0x1010', 'StationNam'],
+                  ['0x0008', '0x0080', 'Institutio'],
+                  ['0x0020', '0x0052', 'FrameOfRef'],
+                  ['0x0028', '0x0008', 'NumberOfFr'],
+                  ['0x3004', '0x000A', 'DoseSummat'],
+                  ['0x3006', '0x0002', 'StructureS'],
+                  ['0x0010', '0x0020', 'SeriesPat'],
+                  ['0x0008', '0x1070', 'OperatorsN'],
+                  ['0x0020', '0x000d', 'StudyInsta']],
+             'DICOMimages':
+                 [['0x0008', '0x0018', 'SOPInstanc'],
+                  ['0x0008', '0x0016', 'SOPClassUI'],
+                  ['0x0020', '0x0013', 'ImageNumbe'],
+                  ['0x0008', '0x0023', 'ImageDate'],
+                  ['0x0008', '0x0033', 'ImageTime'],
+                  ['0x0008', '0x1155', 'Referenced'],
+                  ['0x0018', '0x0086', 'EchoNumber'],
+                  ['0x0028', '0x0008', 'NumberOfFr'],
+                  ['0x0008', '0x0022', 'AcqDate'],
+                  ['0x0008', '0x0032', 'AcqTime'],
+                  ['0x0018', '0x1250', 'ReceivingC'],
+                  ['0x0020', '0x0012', 'AcqNumber'],
+                  ['0x0020', '0x1041', 'SliceLocat'],
+                  ['0x0028', '0x0002', 'SamplesPer'],
+                  ['0x0028', '0x0004', 'PhotoMetri'],
+                  ['0x0028', '0x0010', 'Rows'],
+                  ['0x0028', '0x0011', 'Colums'],
+                  ['0x0028', '0x0030', 'PixelSpaci'],
+                  ['0x0028', '0x0101', 'BitsStored'],
+                  ['0x0028', '0x1052', 'RescaleInt'],
+                  ['0x0028', '0x1053', 'RescaleSlo'],
+                  ['0x0008', '0x0008', 'ImageType'],
+                  ['0x0054', '0x0400', 'ImageID'],
+                  ['0x0010', '0x0020', 'ImagePat'],
+                  ['0x0018', '0x0060', 'KVP'],
+                  ['0x0018', '0x1150', 'ExposureTi'],
+                  ['0x0018', '0x1151', 'TubeCurren'],
+                  ['0x0018', '0x1152', 'Exposure'],
+                  ['0x0018', '0x9345', 'CTDIvol'],
+                  ['0x01F1', '0x1026', 'Pitch'],
+                  ['0x01F1', '0x1027', 'RotationTi'],
+                  ['0x01F1', '0x104A', 'DoseRight'],
+                  ['0x01F1', '0x104B', 'Collimatio'],
+                  ['0x0018', '0x0050', 'SliceThick'],
+                  ['0x0020', '0x0037', 'ImageOrien'],
+                  ['0x0008', '0x0060', 'Modality'],
+                  ['0x0008', '0x103e', 'SeriesDesc'],
+                  ['0x0020', '0x0032', 'ImagePosit'],
+                  ['0x0020', '0x000e', 'SeriesInst']],
+             'DICOMworklist':
+                 [['0x0008', '0x0050', 'AccessionN'],
+                  ['0x0010', '0x0020', 'PatientID'],
+                  ['0x0010', '0x0010', 'PatientNam'],
+                  ['0x0010', '0x0030', 'PatientBir'],
+                  ['0x0010', '0x0040', 'PatientSex'],
+                  ['0x0010', '0x2000', 'MedicalAle'],
+                  ['0x0010', '0x2110', 'ContrastAl'],
+                  ['0x0020', '0x000d', 'StudyInsta'],
+                  ['0x0032', '0x1032', 'ReqPhysici'],
+                  ['0x0032', '0x1060', 'ReqProcDes'],
+                  ['0x0040', '0x0100', '--------'],
+                  ['0x0008', '0x0060', 'Modality'],
+                  ['0x0032', '0x1070', 'ReqContras'],
+                  ['0x0040', '0x0001', 'ScheduledA'],
+                  ['0x0040', '0x0002', 'StartDate'],
+                  ['0x0040', '0x0003', 'StartTime'],
+                  ['0x0040', '0x0006', 'PerfPhysic'],
+                  ['0x0040', '0x0007', 'SchedPSDes'],
+                  ['0x0040', '0x0009', 'SchedPSID'],
+                  ['0x0040', '0x0010', 'SchedStati'],
+                  ['0x0040', '0x0011', 'SchedPSLoc'],
+                  ['0x0040', '0x0012', 'PreMedicat'],
+                  ['0x0040', '0x0400', 'SchedPSCom'],
+                  ['0x0040', '0x0100', '---------'],
+                  ['0x0040', '0x1001', 'ReqProcID'],
+                  ['0x0040', '0x1003', 'ReqProcPri']]}
